package com.lambdazen.bitsy;

import java.util.Map;
import java.util.UUID;

import com.lambdazen.bitsy.store.EdgeBean;
import com.tinkerpop.blueprints.Direction;
import com.tinkerpop.blueprints.Edge;
import com.tinkerpop.blueprints.Vertex;

public class BitsyEdge extends BitsyElement implements Edge, IEdge {
    String label;
    UUID outVertexId;
    UUID inVertexId;
    
    public BitsyEdge(UUID id, Map<String, Object> properties, ITransaction tx, BitsyState state, int version, String label, UUID outVertexId, UUID inVertexId) {
        super(id, properties, tx, state, version);

        if (label == null) {
            throw new IllegalArgumentException("Edge label can not be null"); // Enforced by 2.3.0 test case
        }
        
        this.label = label;
        this.outVertexId = outVertexId;
        this.inVertexId = inVertexId;
    }

    public BitsyEdge(EdgeBean bean, ITransaction tx, BitsyState state) {
        this(bean.getId(), bean.getProperties(), 
                tx, state, bean.getVersion(), bean.getLabel(), bean.getOutVertexId(), bean.getInVertexId());
    }

    public EdgeBean asBean() {
        // The TX is usually not active at this point. So no checks.
        return new EdgeBean(id, properties, version, label, outVertexId, inVertexId, state);
    }
    
    public String toString() {
        return "BitsyEdge(id = " + id + ", properties = " + properties + ", version = " + version + ", state " + state + ", OUT vertex = " + outVertexId + ", IN vertex " + inVertexId + ", label = " + label + ")";
    }
    
    public String getLabel() {
        // There is no Tx validation for label because it is used even after deletion to update indexes, etc. 
        return label;
    }

    public Vertex getVertex(Direction dir) {
        tx.validateForQuery(this);
        
        Vertex ans = tx.getVertex(getVertexId(dir));
        assert (ans != null) : "Edge " + this + " has an invalid vertex in direction " + dir;
        return ans;
    }

    public UUID getInVertexId() {
        return inVertexId;
    }

    public UUID getOutVertexId() {
        return outVertexId;
    }

    public UUID getVertexId(Direction dir) {
        if (dir == Direction.IN) {
            return inVertexId;
        } else if (dir == Direction.OUT) {
            return outVertexId;
        } else {
            throw new IllegalArgumentException("Unsupported direction " + dir);
        }
    }

    public void incrementVersion() {
        this.version++;
    }
    
    public void remove() {
        tx.removeEdge(this);
    }
}
