package com.lambdazen.bitsy;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.UUID;

import com.lambdazen.bitsy.store.VertexBean;
import com.tinkerpop.blueprints.Direction;
import com.tinkerpop.blueprints.Edge;
import com.tinkerpop.blueprints.Vertex;
import com.tinkerpop.blueprints.VertexQuery;
import com.tinkerpop.blueprints.util.DefaultVertexQuery;

public class BitsyVertex extends BitsyElement implements Vertex {
    private static final Direction[] directions = new Direction[] {Direction.OUT, Direction.IN}; 
            
    public BitsyVertex(UUID id, Map<String, Object> properties, ITransaction tx, BitsyState state, int version) {
        super(id, properties, tx, state, version);
    }
    
    public BitsyVertex(VertexBean bean, ITransaction tx, BitsyState state) {
        this(bean.getId(), bean.getProperties(), tx, state, bean.getVersion());
    }

    public Iterable<Edge> getEdges(Direction dir, String... edgeLabels) {
        return tx.getEdges(this, dir, edgeLabels);
    }

    public VertexBean asBean() {
        // The TX is usually not active at this point. So no checks.
        return new VertexBean(id, (TreeMap<String, Object>)properties, version, state);
    }
    
    public Iterable<Vertex> getVertices(final Direction dir, String... edgeLabels) {
        final ArrayList<Vertex> vertices = new ArrayList<Vertex>();
        
        for (Direction myDir : directions) {
            if ((myDir == dir) || (dir == Direction.BOTH)) {
                for (Edge e : getEdges(myDir, edgeLabels)) {
                    Vertex toAdd = e.getVertex(myDir.opposite());
                    vertices.add(toAdd);
                }
            }
        }
        
        // Go through the edges and load the vertices
        return new Iterable<Vertex>() {
            public Iterator<Vertex> iterator() {
                return vertices.iterator();
            }
        };
    }

    public VertexQuery query() {
        return new DefaultVertexQuery(this);
    }

    public void incrementVersion() {
        // It is OK for the version to wrap around MAX_INT
        this.version++;
    }

    public void remove() {
        tx.removeVertex(this);
    }

    @Override
    public Edge addEdge(String label, Vertex inVertex) {
        // Construct the edge with this as the out vertex 
        BitsyEdge edge = new BitsyEdge(UUID.randomUUID(), null, tx, BitsyState.M, 0, label, (UUID)getId(), (UUID)inVertex.getId());

        tx.addEdge(edge);
        
        return edge;
    }
}
