package com.lambdazen.bitsy.index;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import com.lambdazen.bitsy.BitsyErrorCodes;
import com.lambdazen.bitsy.BitsyException;

public class BitsyIndexMap<BeanType, IndexType extends BitsyIndex<BeanType>> {
    List<String> indexNames;
    Map<String, IndexType> indexMap;
    
    public BitsyIndexMap() {
        this.indexMap = new HashMap<String, IndexType>();
        this.indexNames = new ArrayList<String>();
    }

    protected void addKeyIndex(String key, IndexType index) {
        if (indexMap.containsKey(key)) {
            throw new BitsyException(BitsyErrorCodes.INDEX_ALREADY_EXISTS, "Index on vertex key '" + key + "'");
        }

        indexMap.put(key, index);
        indexNames.add(key);
    }
    
    public void dropKeyIndex(String key) {
        indexMap.remove(key);
        indexNames.remove(key);
    }
    
    public Set<String> getIndexedKeys() {
        return new TreeSet<String>(indexNames);
    }

    public Collection<BeanType> get(String key, Object value) {
        IndexType index = indexMap.get(key);
        
        if (index == null) {
            throw new BitsyException(BitsyErrorCodes.MISSING_INDEX, "An index on " + key + " must be created before querying vertices/edges by that key. Defined indexes: " + getIndexedKeys());
        } else {
            Collection<BeanType> ans = index.get(value);
            if (ans == null) {
                return Collections.emptyList();
            } else {
                return ans;
            }
        }
    }
    
    public void add(BeanType bean) {
        for (String keyName : indexNames) {
            IndexType index = indexMap.get(keyName);
            index.add(bean);
        }
    }
    
    public void remove(BeanType bean) {
        if (bean == null) {
            // Nothing to do
            return;
        }
        
        for (String keyName : indexNames) {
            IndexType index = indexMap.get(keyName);
            index.remove(bean);
        }
    }
}
