package com.lambdazen.bitsy.store;

import java.io.Serializable;
import java.util.TreeMap;
import java.util.UUID;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

import com.lambdazen.bitsy.BitsyState;
import com.nesscomputing.uuid.NessUUID;

@JsonPropertyOrder({"id", "v", "s", "p"})
public final class VertexBean implements Serializable {
    private static final long serialVersionUID = -2867517568410927192L;

    private UUID id;
    private int version;
    private BitsyState state;
    private TreeMap<String, Object> properties;

    public VertexBean(UUID uuid, TreeMap<String, Object> properties, int version, BitsyState state) {
        this.id = uuid;
        this.properties = properties;
        this.version = version;
        this.state = state;
    }

    @JsonCreator
    public VertexBean(@JsonProperty("id") String uuidStr, 
            @JsonProperty("p") TreeMap<String, Object> properties, 
            @JsonProperty("v") int version, 
            @JsonProperty("s") BitsyState state) {
        this.id = NessUUID.fromString(uuidStr);// Performs better than UUID.fromString(uuidStr);
        this.version = version;
        this.state = state;
        this.properties = properties;
    }
    
    public String toString() {
        return "VertexBean(id=" + getIdStr() + ", version=" + version + ", state=" + state + ", properties=" + properties + ")"; 
    }

    @JsonIgnore
    public UUID getId() {
        return id;
    }

    @JsonProperty("id")
    public String getIdStr() {
        return NessUUID.toString(id);
    }

    @JsonProperty("p")
    public TreeMap<String, Object> getProperties() {
        return properties;
    }

    @JsonProperty("v")
    public int getVersion() {
        return version;
    }
    
    @JsonProperty("s")
    public BitsyState getState() {
        return state;
    }
}
